import sys
import argparse
import pathlib
import logging
# import pandas as pd
from .config import Config
from . import pipeline, components

logger = logging.getLogger(__name__)


def main(argv=sys.argv):

    epilog = 'Pipeline modules:\n'
    epilog += '\n'.join(f"    {n}" for n in components.pipeline_module_names)
    parser = argparse.ArgumentParser(
        description='Perform CyLinter analysis on a data file.',
        epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        'config', type=path_resolved,
        help='Path to the configuration YAML file'
    )
    parser.add_argument(
        '--module', type=str,
        help='Pipeline module at which to begin processing (see below'
        ' for ordered list of modules)'
    )
    args = parser.parse_args(argv[1:])
    if not validate_paths(args):
        return 1
    if args.module and args.module not in components.pipeline_module_names:
        print(
            f"cylinter: error: argument --module: invalid choice '{args.module}'",
            file=sys.stderr
        )
        return 1

    logging.basicConfig(
        level=logging.INFO,
        format='%(levelname)s: %(message)s'
    )

    logger.info("Reading configuration file")
    config = Config.from_path(args.config)
    create_output_directory(config)

    logger.info("Executing pipeline")
    pipeline.run_pipeline(config, args.module)

    logger.info("Finished")

    return 0


def path_resolved(path_str):
    """Return a resolved Path for a string."""
    path = pathlib.Path(path_str)
    path = path.resolve()
    return path


def validate_paths(args):
    """Validate the Path entries in the argument list."""
    ok = True
    if not args.config.exists():
        print(
            f"Config path does not exist:\n     {args.config}\n",
            file=sys.stderr
        )
        ok = False
    return ok


def create_output_directory(config):
    """Create the output directory structure given the configuration object."""
    config.out_dir.mkdir(parents=True, exist_ok=True)



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    LONG_DESCRIPTION = fh.read()

requires = [
    'PyYAML==5.4',
    'pyarrow==2.0.0',
    'pandas==1.2.0',
    'hdbscan==0.8.26',
    'joblib==0.17.0',  # newly-released joblib==1.0.0 results in issues with parallel processing with larger datasets
    'matplotlib==3.3.3',
    'seaborn==0.11.1',
    'scikit-image==0.18.1',
    'napari==0.3.6',  # points layer doesn't work with napari[all] (v0.4.3)
    'PyQt5==5.15.2',  # must include if not installing napari[all]
    'zarr==2.6.1',
    'natsort==7.1.0',
    'rpy2==3.4.2',
    'bridson==0.1.0',
    'hurry.filesize==0.9',
    'synapseclient==2.0.0',  # v2.2.2 (current version as of 01/25/21 fails to transfer)
]

VERSION = '0.0.15'
DESCRIPTION = 'CyLinter'
AUTHOR = 'Gregory J. Baker'
AUTHOR_EMAIL = 'gregory_baker2@hms.harvard.edu'
LICENSE = 'MIT License'
HOMEPAGE = 'https://github.com/labsyspharm/cylinter'

setup(
    name='cylinter',
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    setup_requires=['cython>=0.29.21', 'numpy>=1.19.5'],
    packages=find_packages(),
    install_requires=requires,
    data_files=[('', ['cylinter/config.yml']),
                ('', ['cylinter/prep_subprocess.sh']),
                ],
    entry_points={
        'console_scripts': [
            'cylinter=cylinter.cylinter:main',
            'prep=cylinter.prep:main',
        ]
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: %s' % LICENSE,
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Scientific/Engineering :: Visualization'
    ],
    python_requires='==3.8.5',
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    license=LICENSE,
    url=HOMEPAGE,
    download_url='%s/archive/v%s.tar.gz' % (HOMEPAGE, VERSION),
    keywords='scripts single cell data science',
    zip_safe=False,
)
